{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWBitMapHolder.pas                                     }
{ Short description  : used by TDFWFoldPanel                                   }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 27.05.98                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-1998                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              : See helpfile                                            }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWBitmapHolder, TDFWLinkChange                          }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWBitMapHolder;

interface

uses Windows, Classes, Graphics;

type
  TDFWBitmapHolder = class;
{ TDFWLinkChange }

  TDFWLinkChange = class(TObject)
  private
    FSender: TDFWBitmapHolder;
    FOnChange: TNotifyEvent;
  public
    destructor Destroy; override;
    procedure Change; dynamic;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Sender: TDFWBitmapHolder read FSender write FSender;
  end;

{ TDFWBitmapHolder }
  TDFWBitmapHolder = class(TComponent)
  private
    { Private-Deklarationen }
    FBitmap : TBitmap;
    FClients : TList;
    procedure OnBitmapChange(Sender : TObject);
  protected
    { Protected-Deklarationen }
    procedure SetBitmap(Value : TBitmap);
    function GetBitmap : TBitmap;
    procedure Change;
  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure RegisterChanges(Value: TDFWLinkChange);
    procedure UnRegisterChanges(Value: TDFWLinkChange);

    procedure DrawOnH   (Dest: TRect ; handle: THandle ; Xsrc, Ysrc : Integer);
    procedure TiledOnH  (Dest: TRect ; handle: THandle);
    procedure StretchOnH(Dest: TRect ; handle: THandle);

    procedure DrawOn   (Dest: TRect ; Canevas: TCanvas ; Xsrc, Ysrc : Integer);
    procedure TiledOn  (Dest: TRect ; Canevas: TCanvas);
    procedure StretchOn(Dest: TRect ; Canevas: TCanvas);

    function isEmpty : Boolean;

  published
    { Published-Deklarationen }
    property Bitmap : TBitmap read GetBitmap write SetBitmap;
  end;

implementation

constructor TDFWBitmapHolder.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    FBitmap := TBitmap.Create;
    FBitmap.OnChange := OnBitmapChange;
    FClients := TList.Create;

end;

destructor TDFWBitmapHolder.Destroy;
begin
  while FClients.Count > 0 do
    UnRegisterChanges(TDFWLinkChange(FClients.Last));

  FBitmap.Free;
  FClients.Free;
  inherited Destroy;
end;

procedure TDFWBitmapHolder.OnBitmapChange(Sender : TObject);
begin  Change;
end;


procedure TDFWBitmapHolder.SetBitmap(Value : TBitmap);
begin
    FBitmap.Assign(Value);
    Change;
end;

function TDFWBitmapHolder.GetBitmap : TBitmap;
begin
   Result := FBitmap;
end;

function TDFWBitmapHolder.isEmpty : Boolean;
begin Result := (FBitmap = NiL) OR (FBitmap.Empty);
end;


procedure TDFWBitmapHolder.Change;
var
  I: Integer;
begin
  for I := 0 to FClients.Count - 1 do
    TDFWLinkChange(FClients[I]).Change;
  //if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TDFWBitmapHolder.UnRegisterChanges(Value: TDFWLinkChange);
var
  I: Integer;
begin
  for I := 0 to FClients.Count - 1 do
    if FClients[I] = Value then
    begin
      Value.Sender := nil;
      FClients.Delete(I);
      Break;
    end;
end;

procedure TDFWBitmapHolder.RegisterChanges(Value: TDFWLinkChange);
begin
  Value.Sender := Self;
  FClients.Add(Value);
end;

procedure TDFWBitmapHolder.DrawOnH(Dest: TRect ; handle: THandle ; Xsrc, Ysrc : Integer);
begin
 BitBlt(Handle, Dest.Left, Dest.Top, Dest.Right, Dest.Bottom,
         FBitmap.Canvas.Handle, Xsrc, Ysrc, SRCCOPY);
end;

procedure TDFWBitmapHolder.TiledOnH(Dest: TRect ; handle: THandle);
var
  Row, Col : Integer;
  NumRows, NumCols: Integer;
  DrawX, DrawY, DrawWidth, DrawHeight: Integer;
//  OldRect : TRect;
begin

  NumRows := Dest.Bottom DIV FBitmap.Height;
  NumCols := Dest.Right DIV FBitmap.Width;
  for Row := 0 to NumRows+1 do
    for Col := 0 to NumCols+1  do
    begin
      DrawX :=  (Col * FBitmap.Width) + Dest.Left;
      DrawY :=  (Row * FBitmap.Height) + Dest.Top;
      if (DrawX + FBitmap.Width) > Dest.Right then
         DrawWidth :=  Dest.Right - DrawX
      else
         DrawWidth :=  FBitmap.Width;

      if (DrawY + FBitmap.Height) > Dest.Bottom then
         DrawHeight :=  Dest.Bottom - DrawY
      else
         DrawHeight := FBitmap.Height;
//      GetClipBox(DC, OldRect);
//      IntersectCliprect(DC, CR.Left, CR.Top, CR.Right, CR.Bottom);
      BitBlt( Handle,
              DrawX, DrawY, DrawWidth, DrawHeight,
              FBitmap.Canvas.Handle,
              0, 0, SRCCOPY);
//      IntersectCliprect(DC, OldRect.Left, OldRect.Top, OldRect.Right, OldRect.Bottom);
    end;
end;

procedure TDFWBitmapHolder.StretchOnH(Dest: TRect ; handle: THandle);
begin
  StretchBlt(Handle, Dest.Top, Dest.Left, Dest.Right, Dest.Bottom,
             FBitmap.Canvas.Handle, 0, 0, FBitmap.Width-1, FBitmap.Height-1, SRCCOPY);
end;

procedure TDFWBitmapHolder.DrawOn(Dest: TRect ; Canevas: TCanvas ; Xsrc, Ysrc : Integer);
begin
 BitBlt(Canevas.Handle, Dest.Left, Dest.Top, Dest.Right, Dest.Bottom,
         FBitmap.Canvas.Handle, Xsrc, Ysrc, SRCCOPY);
end;

procedure TDFWBitmapHolder.TiledOn(Dest: TRect ; Canevas: TCanvas);
begin TiledOnH(Dest, Canevas.Handle);
end;

procedure TDFWBitmapHolder.StretchOn(Dest: TRect ; Canevas: TCanvas);
begin
  StretchBlt(Canevas.Handle, Dest.Top, Dest.Left, Dest.Right, Dest.Bottom,
             FBitmap.Canvas.Handle, 0, 0, FBitmap.Width-1, FBitmap.Height-1, SRCCOPY);
end;

{ TDFWLinkChange }

destructor TDFWLinkChange.Destroy;
begin
  if Sender <> nil then Sender.UnRegisterChanges(Self);
  inherited Destroy;
end;

procedure TDFWLinkChange.Change;
begin
  if Assigned(OnChange) then OnChange(Sender);
end;


end.
