{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWCheckCombo.pas                                       }
{ Short description  :                                                         }
{ Author             : Tsourinakis Antonis                                     }
{ E-Mail             : tsoyran@otenet.gr                                       }
{ Internet           :                                                         }
{ Version            : 1.3                                                     }
{ Date last modified : 2000/04/08                                              }
{ Delphi versions    : tested on D3,D4,D5                                      }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWCheckedComboBox                                       }
{ Classes          :                                                           }
{ Types            : TCHBQuoteStyle                                            }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

unit DFWCheckCombo;

interface

uses
	Windows, Messages, SysUtils,Buttons, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, Menus,CheckLst,extctrls;

const
      ComponentVersion='1.3';
      ComponentAuthor='Tsourinakis Antonis';
      ComponentEMail='tsoyran@otenet.gr';
      ComponentHomePage='';

type

	// added 2000/04/08
	TCHBQuoteStyle   = (qsNone,qsSingle,qsDouble);

	TDFWCheckedComboBox = class(TCustomPanel)
	private
		FCapSelAll,
		FCapDeselAll		: string;
		FEdit						: TEdit;
		FButton					: TSpeedButton;
		FItems     			: TStrings;
		FPrivForm 			: TForm;
		FListBox 				: TCheckListBox;
		FPopup					: TPopupMenu;
		FSelectAll		 	: TMenuItem;
		FDeSelectAll	 	: TMenuItem;
		FNotFocusColor	: TColor;
		FSorted					: Boolean;
		FQuoteStyle  		: TCHBQuoteStyle; // added 2000/04/08
		FCheckedCount		: integer;
		FColumns				: integer;
		FDropDownLines	: integer;
		FOnChange				: TNotifyEvent;
		FOnEnter				: TNotifyEvent;
		FOnExit					: TNotifyEvent;
		procedure SetItems(AItems : TStrings);
		procedure ToggleOnOff( Sender : TObject );
		procedure KeyListBox(Sender: TObject; var Key: Word;Shift: TShiftState);
		procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
		procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
		procedure WMSize(var Message: TWMSize); message WM_SIZE;
		procedure ShowCheckList(Sender: TObject);
		procedure CloseCheckList( Sender : TObject );
		procedure ItemsChange(Sender : TObject);
		procedure SetSorted(Value: Boolean);
		procedure AutoSize;
		procedure AdjustHeight;
		procedure EditOnEnter(Sender: TObject);
		procedure EditOnExit(Sender: TObject);
		procedure SetNotFocusColor(value:TColor);
		procedure SetColumns(value : integer);
		procedure SetChecked(Index: Integer; Checked: Boolean);
		procedure SetDropDownLines(value : integer);
		function  GetChecked(Index: Integer): Boolean;
		procedure Change;
	protected
		procedure CreateWnd; override;
            function GetVersion:string;
            function GetAuthor:string;
            function GetEMail:string;
            function GetHomepage:string;
	public
		constructor Create ( AOwner : TComponent ); override;
		destructor 	Destroy; override;
		procedure 	Clear;
		procedure 	SetUnCheckedAll( Sender: TObject );
		procedure 	SetCheckedAll(  Sender: TObject  );
		function  	IsChecked ( Index: integer ) : boolean;
		function 		GetText : string;
		property 		Checked[Index: Integer]: Boolean
								read GetChecked
								write SetChecked;
		property		CheckedCount: integer read FCheckedCount;
	published
		property  Items: TStrings
							read 	FItems
							write SetItems;
		property  CapSelectAll: String
							read 	FCapSelAll
							write FCapSelAll;
		property  CapDeSelectAll: String
							read 	FCapDeselAll
							write FCapDeselAll;
		property  NotFocusColor: TColor
							read	FNotFocusColor
							write SetNotFocusColor;
		property  Sorted 	: Boolean
							read 	FSorted
							write SetSorted default False;
		property  QuoteStyle	: TCHBQuoteStyle  // added 2000/04/08
							read 	FQuoteStyle
							write FQuoteStyle default qsNone;
		property  Columns	: integer
							read 	FColumns
							write SetColumns default 0;
		property  DropDownLines: integer
							read 	FDropDownLines
							write SetDropDownLines default 6;
		// events
		property 	OnChange: TNotifyEvent
							read 	FOnChange
							write FOnChange;
		property  OnEnter : TNotifyEvent
							read 	FOnEnter
							write FOnEnter;
		property  OnExit: TNotifyEvent
							read 	FOnExit
							write FOnExit;
		// from panel
		property  Ctl3D;
		property  Cursor;
		property  Enabled;
		property  Font;
		property  ParentColor;
		property  ParentCtl3D;
		property  ParentFont;
		property  ParentShowHint;
		property  ShowHint;
		property  TabOrder;
		property  TabStop;
		property  Visible;
		property  OnClick;
		property  OnMouseDown;
		property  OnMouseMove;
		property  OnMouseUp;
		property  OnResize;
		property  OnStartDrag;
            property Version: string read GetVersion;
            property Author: string read GetAuthor;
            property EMail: string read GetEMail;
            property Homepage: string read GetHomepage;
	end;

implementation
{$R *.R32}
const
	Delimit 			=	',';
	MAXSELLENGTH	= 256;
	MINDROPLINES	= 6;
	MAXDROPLINES	= 10;
resourcestring
	sFCapSelAll		= '&Select All';
	sFCapDeselAll	= '&DeSelect All';
	sNoMoreLength = 'There is no room for Selected';

{TDFWCheckedComboBox}
constructor TDFWCheckedComboBox.Create ( AOwner : TComponent );
begin
	inherited Create( AOwner );
	FDropDownLines		:= MINDROPLINES;
	FColumns					:= 0;
	FQuoteStyle 			:= qsNone;	// added 2000/04/08
	FCheckedCount			:= 0;
	FNotFocusColor		:= clWhite;
	Caption 					:= '';
	FCapSelAll				:= sFCapSelAll;
	FCapDeselAll			:= sFCapDeselAll;
	BevelOuter				:= bvLowered;
	Height						:= 24;
	Width 						:= 121;

	FItems      			:= TStringList.Create;
	TStringList(FItems).OnChange := ItemsChange;

	FEdit							:= TEdit.Create(Self);
	FEdit.Parent			:= Self;
	FEdit.ParentColor := false;
	FEdit.color				:= clWhite;
	FEdit.ReadOnly		:= True;

	FButton 					:= TSpeedButton.Create(Self);
	with FButton do
	begin
		Glyph.LoadFromResourceName(HInstance, 'BTNCHECKPOPUP');
		Parent  			:= Self;
		FButton.Width	:= 16;
		FButton.Height:= Self.Height-2;
		FButton.Top		:= Self.Top+2;
		NumGlyphs 		:= 1;
		Parent 				:= Self;
		Layout				:= blGlyphRight;
		OnClick 			:= ShowCheckList;
	end;

	Fedit.Text			:= '';
	FEdit.OnEnter		:= EditOnEnter;
	FEdit.OnExit		:= EditOnExit;

	// Create a form with its contents
	FPrivForm 				:= TForm.Create( self );
	FPrivForm.Color 	:= clWindow;

	// Create CheckListBox
	FListBox 							:= TCheckListBox.Create( FPrivForm );
	FListBox.Parent 			:= FPrivForm;
	FListBox.Ctl3D				:= False;
	FListBox.Columns			:= FColumns;
	FListBox.Align 				:= alClient;
	FListBox.OnClickCheck := ToggleOnOff;
	FListBox.OnKeyDown		:= KeyListBox;
	// Create PopUp
	FPopUp 								:= TPopupMenu.Create( FListBox );
	FSelectAll 						:= TMenuItem.Create( FPopUp );
	FSelectAll.Caption 		:= FCapSelAll;
	FDeSelectAll 					:= TMenuItem.Create( FPopUp );
	FDeSelectAll.Caption 	:= FCapDeselAll;
	FPopUp.Items.Insert( 0, FSelectAll );
	FPopUp.Items.Insert( 1, FDeSelectAll );
	FSelectAll.OnClick 		:= SetCheckedAll;
	FDeSelectAll.OnClick 	:= SetUnCheckedAll;
	FListBox.PopupMenu 		:= FPopUp;
end;

destructor TDFWCheckedComboBox.Destroy;
begin
	FEdit.free;
	FSelectAll.Free;
	FDeSelectAll.Free;
	FPopup.Free;
	FButton.Free;
	FListBox.Free;
	FItems.Free;
	FPrivForm.Free;
	inherited Destroy;
end;
//====================== Show - Close List Box

procedure TDFWCheckedComboBox.ShowCheckList( Sender : TObject );
var	ScreenPoint : TPoint;
begin
	if FButton.tag=1 then  // Jan Verhoeven
	begin
		FButton.tag:=0;
		exit
	end;
	Click;
	if Fcolumns > 1 then
		FDropDownLines := FlistBox.Items.Count div Fcolumns+1;
	if FDropDownLines < MINDROPLINES then
		FDropDownLines := MINDROPLINES;
	if FDropDownLines > MAXDROPLINES then
		FDropDownLines := MAXDROPLINES;

	// Assign Form coordinate and show
	ScreenPoint    				:= Parent.ClientToScreen( Point( self.Left, self.Top+self.Height ) );
	FSelectAll.Caption 		:= FCapSelAll;
	FDeSelectAll.Caption 	:= FCapDeselAll;
	with FPrivForm do
	begin
		Font 					:= self.Font;
		Left  				:= ScreenPoint.X;
		Top   				:= ScreenPoint.Y;
		Width 				:= self.Width;
		Height 				:= ( FDropDownLines * FlistBox.ItemHeight+4{ FEdit.Height });
		BorderStyle 	:= bsNone;
		OnDeactivate	:= CloseCheckList;
	end;
	FPrivForm.Show;
end;

procedure TDFWCheckedComboBox.CloseCheckList( Sender : TObject );
var pt:TPoint;
begin
// code added by Jan Verhoeven
// check if the mouse is over the combobox button
//	pt:=mouse.CursorPos;
//  this doesn't work on delphi 3
	GetCursorPos(pt);
	pt:=FButton.ScreenToClient(pt);
	with Fbutton do
	begin
		if (pt.x>0) and (pt.x<width) and (pt.y>0) and (pt.y<height)
			then tag:=1
			else tag:=0
	end;
	FPrivForm.Close;
end;

//===========================================
// exanines if string (part) exist in string (source)
// 		where source is in format part1[,part2]
function PartExist(const part, source:string):Boolean;
var m :integer;
	 temp1,temp2 :string;
begin
	temp1  := Copy(Source,1,MAXSELLENGTH);
	Result := part=temp1;
	while not result do
	begin
		m := Pos(Delimit,temp1);
		if m>0 then
			temp2 := Copy(temp1,1,m-1)
		else
			temp2 := temp1;
		Result := part=temp2;
		if (Result) or (m=0) then break;
		temp1 := Copy(temp1,m+1,MAXSELLENGTH);
	end;
end;

{
	removes a string (part) from another string (source)
	when source is in format part1[,part2]
}
function RemovePart(const part, source :string):string;
var
	lp,p :integer;
	s1,s2:string;
begin
	s2 := source;
	lp := Length(part);
	 p := Pos(part,source);
	if p>0 then
	begin
		s1 := '';
		if p<>1 then
			s1 := Copy(source,1,p-1);
		s2 := s1 + Copy(source,p+lp+1,MAXSELLENGTH);
		lp := Length(s2);
		if (lp>0) and (s2[lp]=Delimit) then
			s2:= Copy(s2,1,lp-1);
	end;
	result := s2;
end;

function Add(const sub:string;var str:string):boolean;
begin
	result := false;
	if length(str)+length(sub)+1>= MAXSELLENGTH then
	begin
		ShowMessage(sNoMoreLength);
		exit;
	end;
	if str = '' then
	begin
		str := sub;
		result := true;
	end
	else
		if not PartExist(sub,str) then
		begin
			str := str+Delimit+ sub;
			result := true;
		end;
end;

function Remove(const sub:string; var str:string):boolean;
	var temp:string;
begin
	result := false;
	if str <> '' then
	begin
		temp 	:= RemovePart(sub,str);
		if temp<>str then
		begin
			str		:= temp;
			result := true;
		end;
	end;
end;

procedure TDFWCheckedComboBox.ToggleOnOff( Sender : TObject );
var s: string;
begin
	s:= fEdit.text;
	if FListBox.Checked[FListBox.itemindex] then
	begin
		if Add(FListBox.Items[FListBox.itemindex],s) then
			FCheckedCount := FCheckedCount + 1
	end
	else
		if Remove(FListBox.Items[FListBox.itemindex],s) then
			FCheckedCount := FCheckedCount - 1;
	fEdit.text:= s;
	Change
end;

procedure TDFWCheckedComboBox.KeyListBox(Sender: TObject; var Key: Word;Shift: TShiftState);
begin
	if key = VK_ESCAPE then
		FPrivForm.Close
	else
		inherited
end;

// added 2000/04/08
function GetFormatedText(kind:TCHBQuoteStyle;str:string):string;
var s : string;
begin
	result := str;
	if length(str)>0 then
	begin
		s := str;
		case kind of
			qsSingle	: result :=
					''''+
					StringReplace(S, ',', ''',''',[rfReplaceAll])+
					'''';
			qsDouble 	: result :=
					'"'+
					StringReplace(S, ',', '","',[rfReplaceAll])+
					'"';
		end;
	end;
end;

function TDFWCheckedComboBox.GetText : string;
begin
	if FQuoteStyle = qsNone then
		result := FEdit.Text
	else
		result := GetFormatedText(FQuoteStyle,FEdit.Text);
end;

//========================== CheckListBox
procedure TDFWCheckedComboBox.SetDropDownLines(value : integer);
begin
	if FDropDownLines<>value then
		if (value>=MINDROPLINES) and (value<=MAXDROPLINES) then
			FDropDownLines := value;
end;

procedure TDFWCheckedComboBox.SetColumns(value : integer);
begin
	if Fcolumns<>value then
	begin
		Fcolumns := value;
		FListBOx.Columns := Fcolumns;
	end;
end;

procedure TDFWCheckedComboBox.SetCheckedAll( Sender : TObject );
var i:integer;
		s: string;
begin
	s:= '';
	for i:=0 to FListBox.Items.Count -1 do
	begin
		if not FListBox.Checked[i] then
		begin
			FListBox.Checked[i]:= true;
		end;
		if i=0 then
			s := FListBox.Items[i]
		else
			s := s+','+FListBox.Items[i];
	end;
	FEdit.text:= s;
	FCheckedCount := FListBox.Items.Count;
	FEdit.Repaint;
	change;
end;

procedure TDFWCheckedComboBox.SetUnCheckedAll( Sender: TObject );
var
	i : integer;
begin
	FCheckedCount := 0;
	with FListBox do
		begin
			for i := 0 to Items.Count-1 do
				if Checked[i] then
					Checked[i] := false;
		end;
	 FEdit.Text:= '';
	change;
end;

function TDFWCheckedComboBox.IsChecked( Index: integer ) : boolean;
begin
	result := FListBox.Checked[ Index ];
end;

procedure TDFWCheckedComboBox.SetChecked(Index: Integer; Checked: Boolean);
var
	s:string;
	ok:boolean;
begin
	if index<FlistBox.Items.Count then
	begin
		s:= fEdit.text;
		ok:= false;
		if not FListBox.Checked[Index] and checked then
		begin
			if Add(FListBox.Items[Index],s) then
			begin
				FCheckedCount := FCheckedCount + 1;
				ok := true;
			end;
		end
		else
		if FListBox.Checked[Index] and not checked then
			if Remove(FListBox.Items[Index],s) then
			begin
				FCheckedCount := FCheckedCount - 1;
				ok := true;
			end;
		if ok then
		begin
			FListBox.Checked[Index]:= Checked;
			fEdit.text:= s;
			Change
		end;
	end;
	{
		if FListBox.Checked[Index] not Checked then
		begin
			FListBox.Checked[Index]:= Checked;
			if Add(FListBox.Items[index],s) then
				FCheckedCount := FCheckedCount + 1
	 }
end;

Function TDFWCheckedComboBox.GetChecked(Index: Integer): Boolean;
begin
	if index<FlistBox.Items.Count then
		Result := FListBox.Checked[Index]
	else
		Result := false;
end;

//===========		For CheckListBox Items
procedure TDFWCheckedComboBox.SetItems(AItems : TStrings);
begin
	FItems.Assign(AItems);
end;

procedure TDFWCheckedComboBox.ItemsChange(Sender : TObject);
begin
	FlistBox.Clear;
	FEdit.Text := '';
	FlistBox.Items.Assign(FItems);
end;

//===========		 Auto Sizing routines
procedure TDFWCheckedComboBox.CMEnabledChanged(var Message: TMessage);
begin
	inherited;
	FButton.Enabled := Enabled;
	FEdit.Enabled   := Enabled;
end;

procedure TDFWCheckedComboBox.AutoSize;
begin
	AdjustHeight;
	FButton.Height	:= Height-2;
	FEdit.Height		:= Height;
	FEdit.width			:= Width - FButton.Width-3;
	FButton.Left		:= FEdit.width+1;
end;

procedure TDFWCheckedComboBox.AdjustHeight;
var
	DC: HDC;
	SaveFont: HFont;
	I: Integer;
	SysMetrics, Metrics: TTextMetric;
begin
	DC := GetDC(0);
	GetTextMetrics(DC, SysMetrics);
	SaveFont := SelectObject(DC, Font.Handle);
	GetTextMetrics(DC, Metrics);
	SelectObject(DC, SaveFont);
	ReleaseDC(0, DC);
	if NewStyleControls then
	begin
		if Ctl3D then I := 8 else I := 6;
		I := GetSystemMetrics(SM_CYBORDER) * I;
	end else
	begin
		I := SysMetrics.tmHeight;
		if I > Metrics.tmHeight then I := Metrics.tmHeight;
		I := I div 4 + GetSystemMetrics(SM_CYBORDER) * 4;
	end;
	Height := Metrics.tmHeight + I;
end;

procedure TDFWCheckedComboBox.CreateWnd;
begin
	inherited;
	AutoSize;
end;

procedure TDFWCheckedComboBox.WMSize(var Message: TWMSize);
begin
	inherited;
	AutoSize;
end;

procedure TDFWCheckedComboBox.CMFontChanged(var Message: TMessage);
begin
	inherited;
	AutoSize;
	invalidate;
end;
//=====================
procedure TDFWCheckedComboBox.EditOnEnter;
begin
	FEdit.Color		:= clWhite;
	if Assigned(FOnEnter) then FOnEnter(Self);
end;

procedure TDFWCheckedComboBox.EditOnExit;
begin
	Fedit.Color		 := FNotFocusColor;
	if Assigned(FOnExit) then FOnExit(Self);
end;

procedure TDFWCheckedComboBox.SetNotFocusColor(value:TColor);
begin
	if FNotFocusColor <> Value then
	begin
		FNotFocusColor := Value;
		Fedit.Color		 := value;
	end;
end;

procedure TDFWCheckedComboBox.Change;
begin
	if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TDFWCheckedComboBox.Clear;
begin
	FEdit.Text := '';
	FItems.Clear;
	FListBox.Clear;
end;

procedure TDFWCheckedComboBox.SetSorted(Value: Boolean);
begin
	if FSorted <> Value then
	begin
		FSorted := Value;
		TStringList(FItems).Sorted := FSorted;
	end;
end;

function TDFWCheckedComboBox.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWCheckedComboBox.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWCheckedComboBox.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWCheckedComboBox.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
