{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWFindFile.pas                                         }
{ Short description  :                                                         }
{ Author             : Kambiz R. Khojasteh                                     }
{ E-Mail             : khojasteh@www.dci.co.ir                                 }
{ Internet           : http://www.crosswinds.net/~khojasteh/                   }
{ Version            : 2.2                                                     }
{ Date last modified : April 14, 2000                                          }
{ Delphi versions    : Delphi 3 and up                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWFileFind                                              }
{ Classes          : EFindFileError                                            }
{ Types            : TFoundEvent, TNewFolderEvent, TFileAttribute,             }
{                    TFileAttributes, TSearchInfo                              }
{ Constants        : BufferSize, PathDelimiter, DirectoryAttr,                 }
{                    InvalidLocation, InvalidFilename                          }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWFindFile;

interface

uses
  {$IFDEF WIN32} Windows {$ELSE} WinTypes, WinProcs {$ENDIF}, Messages,
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
      ComponentVersion='2.2';
      ComponentAuthor='Kambiz R. Khojasteh';
      ComponentEMail='khojasteh@www.dci.co.ir';
      ComponentHomePage='http://www.crosswinds.net/~khojasteh/';

type

  EFindFileError = class(Exception);

  TFoundEvent = procedure (Sender: TObject; Folder: String;
    var FileInfo: TSearchRec) of object;

  TNewFolderEvent = procedure (Sender: TObject; Folder: String;
    var IgnoreFolder: Boolean) of object;

  TFileAttribute = (ffArchive, ffReadonly, ffHidden, ffSystem, ffDirectory);
  TFileAttributes = set of TFileAttribute;

  {TSearchInfo holds all search parameters. This gives us ability to}
  {change TFinFile properties while it is searching.}
  TSearchInfo = record
     IncludeFiles: TStringList;
     ExcludeFiles: TStringList;
     IncSubfolders: Boolean;
     Attr: TFileAttributes;
     ExactAttr: Boolean;
     {$IFDEF WIN32}
     CreatedBefore: TDateTime;
     CreatedAfter: TDateTime;
     AccessedBefore: TDateTime;
     AccessedAfter: TDateTime;
     {$ENDIF}
     ModifiedBefore: TDateTime;
     ModifiedAfter: TDateTime;
     SizeMin: LongInt;
     SizeMax: LongInt;
     Text: PChar;
     IgnoreCase: Boolean;
  end;

  TDFWFindFile = class(TComponent)
  private
    fFilename: String;
    fLocation: String;
    fIncludeFiles: TStringList;
    fExcludeFiles: TStringList;
    fSubfolders: Boolean;
    fModifiedBefore: TDateTime;
    fModifiedAfter: TDateTime;
    fSizeMin: LongInt;
    fSizeMax: LongInt;
    fAttributes: TFileAttributes;
    fExactAttr: Boolean;
    {$IFDEF WIN32}
    fCreatedBefore: TDateTime;
    fCreatedAfter: TDateTime;
    fAccessedBefore: TDateTime;
    fAccessedAfter: TDateTime;
    {$ENDIF}
    fContaining: String;
    fIgnoreCase: Boolean;
    fAborted: Boolean;
    fBusy: Boolean;
    fOnFound: TFoundEvent;
    fOnNewFolder: TNewFolderEvent;
    fOnComplete: TNotifyEvent;
    SI: TSearchInfo;
    {$IFDEF WIN32}
    fThread: TThread;
    fThreaded: Boolean;
    fThreadPriority: TThreadPriority;
    procedure ThreadTerminated(Sender: TObject);
    procedure SetThreadPriority(Value: TThreadPriority);
    {$ENDIF}
    procedure SetFilename(Value: String);
    procedure SetLocation(Value: String);
    procedure SetIncludeFiles(Value: TStringList);
    procedure SetExcludeFiles(Value: TStringList);
  protected
    {$IFDEF WIN32}
    procedure DoThreadedSearch; dynamic;
    {$ENDIF}
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure DoSearch; dynamic;
    procedure DoFound(Folder: String; var FileInfo: TSearchRec); virtual;
    function DoNewFolder(const Folder: String): Boolean; virtual;
    procedure DoComplete; virtual;
    function AcceptFile(const Folder: String; const SR: TSearchRec): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
    procedure Abort;
    property Busy: Boolean read fBusy;
    property Aborted: Boolean read fAborted;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Filename: String read fFilename write SetFilename;
    property Location: String read fLocation write SetLocation;
    property IncludeFiles: TStringList read fIncludeFiles write SetIncludeFiles;
    property ExcludeFiles: TStringList read fExcludeFiles write SetExcludeFiles;
    property Subfolders: Boolean read fSubfolders write fSubfolders default True;
    property Attributes: TFileAttributes read fAttributes write fAttributes default
      [ffArchive, ffReadonly, ffHidden, ffSystem];
    property ExactAttribute: Boolean read fExactAttr write fExactAttr default False;
    property ModifiedBefore: TDateTime read fModifiedBefore write fModifiedBefore;
    property ModifiedAfter: TDateTime read fModifiedAfter write fModifiedAfter;
    {$IFDEF WIN32}
    property CreatedBefore: TDateTime read fCreatedBefore write fCreatedBefore;
    property CreatedAfter: TDateTime read fCreatedAfter write fCreatedAfter;
    property AccessedBefore: TDateTime read fAccessedBefore write fAccessedBefore;
    property AccessedAfter: TDateTime read fAccessedAfter write fAccessedAfter;
    {$ENDIF}
    property SizeMin: LongInt read fSizeMin write fSizeMin default 0;
    property SizeMax: LongInt read fSizeMax write fSizeMax default 0;
    property Containing: String read fContaining write fContaining;
    property IgnoreCase: Boolean read fIgnoreCase write fIgnoreCase default True;
    {$IFDEF WIN32}
    property Threaded: Boolean read fThreaded write fThreaded default False;
    property ThreadPriority: TThreadPriority read fThreadPriority write
      SetThreadPriority default tpNormal;
    {$ENDIF}
    property OnFound: TFoundEvent read fOnFound write fOnFound;
    property OnNewFolder: TNewFolderEvent read fOnNewFolder write fOnNewFolder;
    property OnComplete: TNotifyEvent read fOnComplete write fOnComplete;
  end;

function RemoveTrailingBackslash(S: String): String;
function FileMatches(Path, Mask: String): Boolean;

implementation

uses
  FileCtrl;

const
  BufferSize = 4096;
  PathDelimiter = ';';
  SearchAttr = faAnyFile and not faVolumeID;
  InvalidLocation = '%s' + #10#13 + 'location cannot to be contained wildcards';
  InvalidFilename = '%s' + #10#13 + 'Invalid filename specified';

{$IFDEF WIN32}
type
  TSearchThread = class(TThread)
  private
    Ignored: Boolean;
    Folder: String;
    FoundSR: TSearchRec;
    FindFile: TDFWFindFile;
    procedure NewFolderEntered;
    procedure FileFound;
  protected
    constructor Create(AFindFile: TDFWFindFile);
    procedure Execute; override;
  end;
{$ENDIF}

function RemoveTrailingBackslash(S: String): String;
begin
  if (Length(S) > 1) and (S[Length(S)] = '\') and (S[Length(S)-1] <> ':') then
    Result := Copy(S, 1, Length(S)-1)
  else
    Result := S;
end;

function FileMatches(Path, Mask: String): Boolean;
var
  Fi, Mi: Integer;
  FilePath, FileName: String;
  MaskPath, MaskName: String;
begin
  Result := False;
  FilePath := UpperCase(ExtractFilePath(Path));
  FileName := UpperCase(ExtractFileName(Path));
  MaskPath := UpperCase(ExtractFilePath(Mask));
  MaskName := UpperCase(ExtractFileName(Mask));
  {Checkes file path part, if mask contains path}
  if Length(MaskPath) > 0 then
  begin
    {Checkes drive, if mask contains drive}
    if (Length(MaskPath) >= 2) and (MaskPath[2] = ':') then
    begin
      if not (MaskPath[1] in ['*','?']) and (MaskPath[1] <> FilePath[1]) then
        Exit; {Not Matched}
      {Removes drive part from the path}
      MaskPath := Copy(MaskPath, 3, Length(MaskPath)-2);
    end;
    {Checkes directory}                  { |-> excludes drive length}
    if Length(MaskPath) > Length(FilePath)-2 then
      Exit {Not Matched, Mask is latger than file path}
    else
    begin
      {we check inner directories}
      FilePath := Copy(FilePath, Length(FilePath)-Length(MaskPath)+1, Length(MaskPath));
      if FilePath <> MaskPath then
        Exit {Not Matched}
    end;
  end;
  {Checkes file name part, if mask contains filename}
  if Length(MaskName) > 0 then
  begin
    Mi := 1;
    Fi := 1;
    while (Mi <= Length(MaskName)) and (Fi <= Length(FileName)) do
    begin
      if (FileName[Fi] = MaskName[Mi]) or ((FileName[Fi] <> '.') and (MaskName[Mi] = '?')) then
      begin
        Inc(Mi);
        Inc(Fi);
      end
      else if FileName[Fi] = '.' then
        if MaskName[Mi] = '?' then
          Inc(Mi)
        else if MaskName[Mi] = '*' then
          while (Mi <= Length(MaskName)) and (MaskName[Mi] <> '.') do
            Inc(Mi)
        else
          Exit {Not matched}
      else if MaskName[Mi] = '*' then
        Inc(Fi)
      else
        Exit; {Not matched}
    end;
    while (Mi <= Length(MaskName)) and (MaskName[Mi] in ['?', '*']) do
     Inc(Mi);
    if (Mi > Length(MaskName)) and (Fi > Length(FileName)) then
      Result := True; {Matched}
  end
  else
    Result := True; {Matched}
end;

constructor TDFWFindFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fAttributes := [ffArchive, ffReadonly, ffHidden, ffSystem];
  fExactAttr := False;
  fFilename := '*.*';
  fIncludeFiles := TStringList.Create;
  fExcludeFiles := TStringList.Create;
  fSubfolders := True;
  fIgnoreCase := True;
  fBusy := False;
  {$IFDEF WIN32}
  fThread := nil;
  fThreaded := False;
  fThreadPriority := tpNormal;
  {$ENDIF}
end;

destructor TDFWFindFile.Destroy;
begin
  {$IFDEF WIN32}
  if Assigned(fThread) then
  begin
    fThread.Terminate;
    fThread.WaitFor;
  end;
  {$ENDIF}
  fIncludeFiles.Free;
  fExcludeFiles.Free;
  inherited Destroy;
end;

procedure TDFWFindFile.SetFilename(Value: String);
begin
  if ExtractFileName(Value) = Value then
    fFilename := Value
  else
    raise EFindFileError.CreateFmt(InvalidFilename, [Value]);
end;

procedure TDFWFindFile.SetLocation(Value: String);
begin
  if (Pos('*', Value) = 0) and (Pos('?', Value) = 0) then
    fLocation := Value
  else
    raise EFindFileError.CreateFmt(InvalidLocation, [Value]);
end;

procedure TDFWFindFile.SetIncludeFiles(Value: TStringList);
begin
  if Assigned(Value) then
    fIncludeFiles.Assign(Value)
  else
    fIncludeFiles.Clear;
end;

procedure TDFWFindFile.SetExcludeFiles(Value: TStringList);
begin
  if Assigned(Value) then
    fExcludeFiles.Assign(Value)
  else
    fExcludeFiles.Clear;
end;

{$IFDEF WIN32}
procedure TDFWFindFile.SetThreadPriority(Value: TThreadPriority);
begin
  fThreadPriority := Value;
  if Assigned(fThread) then
    fThread.Priority := Value;
end;
{$ENDIF}

procedure TDFWFindFile.Execute;

  procedure AddToIncludeList(Loc, Filename: String);
  var
    Path: String;
    P, I: Integer;
    Duplicated: Boolean;
  begin
    if fFilename = '' then
      Filename := '*.*'
    else if Pos('.', Filename) = 0 then
      Filename := Filename + '.*';
    repeat
      P := Pos(PathDelimiter, Loc);
      if P > 0 then
      begin
        Path := Copy(Loc, 1, P-1);
        Delete(Loc, 1, P);
      end
      else
      begin
        Path := Loc;
        Loc := '';
      end;
      if Path = '' then
        Path := ExpandFileName('.')
      else
        Path := ExpandFileName(Path);
      if Path[Length(Path)] <> '\' then
        Path := Path + '\';
      Path := Path + Filename;
      Duplicated := False;
      for I := 0 to SI.IncludeFiles.Count-1 do
        if CompareText(SI.IncludeFiles[I], Path) = 0 then
        begin
          Duplicated := True;
          Break;
        end;
      if not Duplicated then
        SI.IncludeFiles.Add(Path);
    until (Length(Loc) = 0) or (Loc = PathDelimiter);
  end;

begin
  if fBusy then Exit;
  fBusy := True;
  {Sets file attributes}
  SI.Attr := fAttributes;
  SI.ExactAttr := fExactAttr;
  {Sets included files}
  SI.IncludeFiles := TStringList.Create;
  if fIncludeFiles.Count > 0 then
    SI.IncludeFiles.Assign(fIncludeFiles)
  else {Addes filename and location to included files}
    AddToIncludeList(fLocation, fFileName);
  SI.IncSubfolders := fSubfolders;
  {Sets excluded files}
  SI.ExcludeFiles := TStringList.Create;
  SI.ExcludeFiles.Assign(fExcludeFiles);
  {Sets date ranges}
  {$IFDEF WIN32}
  SI.AccessedBefore := fAccessedBefore;
  SI.AccessedAfter := fAccessedAfter;
  SI.CreatedBefore := fCreatedBefore;
  SI.CreatedAfter := fCreatedAfter;
  {$ENDIF}
  SI.ModifiedBefore := fModifiedBefore;
  SI.ModifiedAfter := fModifiedAfter;
  {Sets files size ranges}
  SI.SizeMin := fSizeMin;
  SI.SizeMax := fSizeMax;
  {Sets containing text}
  if fContaining <> '' then
  begin
    SI.IgnoreCase := fIgnoreCase;
    GetMem(SI.Text, Length(fContaining)+1);
    StrPCopy(SI.Text, fContaining);
  end;
  {Starts search}
  fAborted := False;
  {$IFDEF WIN32}
  if fThreaded then
    DoThreadedSearch
  else
  {$ENDIF}
    DoSearch;
end;

procedure TDFWFindFile.Abort;
begin
  fAborted := True;
  {$IFDEF WIN32}
  if Assigned(fThread) then
    fThread.Terminate;
  repeat
    Application.ProcessMessages
  until not Busy;
  {$ENDIF}
end;

function TDFWFindFile.AcceptFile(const Folder: String; const SR: TSearchRec): Boolean;

  function ValidFileAttr: Boolean;
  var
    FileAttr: TFileAttributes;
  begin
    Result := False;
    FileAttr := [];
    if (SR.Attr and faArchive) <> 0 then Include(FileAttr, ffArchive);
    if (SR.Attr and faReadonly) <> 0 then Include(FileAttr, ffReadonly);
    if (SR.Attr and faHidden) <> 0 then Include(FileAttr, ffHidden);
    if (SR.Attr and faSysFile) <> 0 then Include(FileAttr, ffSystem);
    if (SR.Attr and faDirectory) <> 0 then Include(FileAttr, ffDirectory);
    if SI.ExactAttr then
    begin
      if FileAttr = SI.Attr then
        Result := True;
    end
    else if (FileAttr = []) or ((FileAttr * SI.Attr) <> []) then
      Result := True;
  end;

  function DateBetween(aDate, Before, After: TDateTime): Boolean;
  begin
    Result := True;
    if Before <> 0 then
      if Frac(Before) = 0 then      { Checks date only }
        Result := Result and (Int(aDate) <= Before)
      else if Int(Before) = 0 then  { Checks time only }
        Result := Result and (Frac(aDate) <= Before)
      else                          { Checks date and time }
        Result := Result and (aDate <= Before);
    if After <> 0 then
      if Frac(After) = 0 then       { Checks date only }
        Result := Result and (Int(aDate) >= After)
      else if Int(After) = 0 then   { Checks time only }
        Result := Result and (Frac(aDate) >= After)
      else                          { Checks date and time }
        Result := Result and (aDate >= After);
  end;

  function FileContainsText: Boolean;
  var
    Stream: TFileStream;
    Buffer: array[0..BufferSize-1] of Char;
    I, TextLen, ReadLen: LongInt;
    {$IFDEF WIN32}
    Compare: function(const Str1, Str2: PChar; MaxLen: Cardinal): Integer;
    {$ELSE}
    Compare: function(Str1, Str2: PChar; MaxLen: Word): Integer;
    {$ENDIF}
  begin
    Result := False;
    {$IFDEF WIN32}
    if SI.IgnoreCase then Compare := @StrLIComp else Compare := @StrLComp;
    {$ELSE}
    if SI.IgnoreCase then Compare := StrLIComp else Compare := StrLComp;
    {$ENDIF}
    TextLen := StrLen(SI.Text);
    Stream := TFileStream.Create(Folder + SR.Name, fmOpenRead or fmShareDenyNone);
    try try
      repeat
        ReadLen := Stream.Read(Buffer, BufferSize);
        for I := 0 to ReadLen - TextLen do
          if Compare(SI.Text, @Buffer[I], TextLen) = 0 then
          begin
            Result := True;
            Break;
          end;
      until Stream.Position >= Stream.Size;
    except
      {Ignores exceptions}
    end;
    finally
      Stream.Free;
    end;
  end;

var
  I: Integer;
  TheDate: TDateTime;
  {$IFDEF WIN32}
  SystemTime: TSystemTime;
  FileTime: TFileTime;
  {$ENDIF}
begin
  Result := False;
  {Checkes file attributes}
  if not ValidFileAttr then Exit;
  {Checkes file size ranges}
  if (SR.Attr and faDirectory) = 0 then
  begin
    if (SI.SizeMin <> 0) and (SR.Size < SI.SizeMin) then Exit;
    if (SI.SizeMax <> 0) and (SR.Size > SI.SizeMax) then Exit;
  end;
  {Checkes file date ranges}
  {$IFDEF WIN32}
  if (SI.CreatedBefore <> 0) or (SI.CreatedAfter <> 0) then
  begin
    FileTimeToLocalFileTime(SR.FindData.ftCreationTime, FileTime);
    FileTimeToSystemTime(FileTime, SystemTime);
    TheDate := SystemTimeToDateTime(SystemTime);
    if not DateBetween(TheDate, SI.CreatedBefore, SI.CreatedAfter) then Exit;
  end;
  if (SI.ModifiedBefore <> 0) or (SI.ModifiedAfter <> 0) then
  begin
    FileTimeToLocalFileTime(SR.FindData.ftLastWriteTime, FileTime);
    FileTimeToSystemTime(FileTime, SystemTime);
    TheDate := SystemTimeToDateTime(SystemTime);
    if not DateBetween(TheDate, SI.ModifiedBefore, SI.ModifiedAfter) then Exit;
  end;
  if (SI.AccessedBefore <> 0) or (SI.AccessedAfter <> 0) then
  begin
    FileTimeToLocalFileTime(SR.FindData.ftLastAccessTime, FileTime);
    FileTimeToSystemTime(FileTime, SystemTime);
    TheDate := SystemTimeToDateTime(SystemTime);
    if not DateBetween(TheDate, SI.AccessedBefore, SI.AccessedAfter) then Exit;
  end;
  {$ELSE}
  if (SI.ModifiedBefore <> 0) or (SI.ModifiedAfter <> 0) then
  begin
    TheDate := FileDateToDateTime(SR.Time);
    if not DateBetween(TheDate, SI.ModifiedBefore, SI.ModifiedAfter) then Exit;
  end;
  {$ENDIF}
  {Checkes exclude file list}
  for I := 0 to SI.ExcludeFiles.Count-1 do
    if FileMatches(Folder + SR.Name, SI.ExcludeFiles[I]) then Exit;
  {Checkes containing text}
  if (SI.Text <> nil) and ((SR.Attr and faDirectory) = 0) then
    Result := FileContainsText
  else
    Result := True;
end;

procedure TDFWFindFile.DoFound(Folder: String; var FileInfo: TSearchRec);
begin
  if Assigned(fOnFound) and not (csDestroying in ComponentState) then
    fOnFound(Self, RemoveTrailingBackslash(Folder), FileInfo);
end;

function TDFWFindFile.DoNewFolder(const Folder: String): Boolean;
var
  Ignored: Boolean;
begin
  Ignored := False;
  if Assigned(fOnNewFolder) and not (csDestroying in ComponentState) then
    fOnNewFolder(Self, RemoveTrailingBackslash(Folder), Ignored);
  Result := not Ignored;
end;

procedure TDFWFindFile.DoComplete;
begin
  if SI.IncludeFiles <> nil then
  begin
    SI.IncludeFiles.Free;
    SI.IncludeFiles := nil;
  end;
  if SI.ExcludeFiles <> nil then
  begin
    SI.ExcludeFiles.Free;
    SI.ExcludeFiles := nil;
  end;
  if SI.Text <> nil then
  begin
    FreeMem(SI.Text, StrLen(SI.Text)+1);
    SI.Text := nil;
  end;
  fBusy := False;
  if Assigned(fOnComplete) and not (csDestroying in ComponentState) then
    fOnComplete(Self);
end;

procedure TDFWFindFile.DoSearch;

  procedure SearchIn(Loc: String; var FileMask: String);
  var
    SR: TSearchRec;
    Path: String;
  begin
    if not DoNewFolder(Loc) then Exit;
    Path := Loc + FileMask;
    {Searches in current folder}
    if not fAborted and (FindFirst(Path, SearchAttr, SR) = 0) then
      repeat
        if (SR.Name[1] <> '.') and AcceptFile(Loc, SR) then
          DoFound(Loc, SR);
      until Aborted or (FindNext(SR) <> 0);
    FindClose(SR);
    {Scans sub folders}
    if not Aborted and SI.IncSubfolders then
    begin
      Path := Loc + '*.*';
      if not Aborted and (FindFirst(Path, SearchAttr, SR) = 0) then
        repeat
          if ((SR.Attr and faDirectory) <> 0) and (SR.Name[1] <> '.') then
            SearchIn(Loc + SR.Name + '\', FileMask);
        until Aborted or (FindNext(SR) <> 0);
      FindClose(SR);
    end;
  end;

var
  I: Integer;
  Folder, Filename: String;

begin
  try
    try
      for I := 0 to SI.IncludeFiles.Count-1 do
      begin
        Folder := ExpandFileName(ExtractFilePath(SI.IncludeFiles[I]));
        Filename := ExtractFileName(SI.IncludeFiles[I]);
        SearchIn(Folder, Filename);
        if fAborted then Break;
      end;
    except
      fAborted := True;
    end;
  finally
    DoComplete;
  end;
end;

{$IFDEF WIN32}

procedure TDFWFindFile.DoThreadedSearch;
begin
  fThread := TSearchThread.Create(Self);
end;

procedure TDFWFindFile.ThreadTerminated(Sender: TObject);
begin
  fThread := nil;
  DoComplete;
end;

constructor TSearchThread.Create(AFindFile: TDFWFindFile);
begin
  FindFile := AFindFile;
  OnTerminate := FindFile.ThreadTerminated;
  FreeOnTerminate := True;
  inherited Create(False);
end;

procedure TSearchThread.NewFolderEntered;
begin
  Ignored := not FindFile.DoNewFolder(Folder);
end;

procedure TSearchThread.FileFound;
begin
  FindFile.DoFound(Folder, FoundSR);
end;

procedure TSearchThread.Execute;

  procedure SearchIn(Loc: String; var FileMask: String);
  var
    SR: TSearchRec;
    Path: String;
  begin
    Folder := Loc;
    Synchronize(NewFolderEntered);
    if Ignored then Exit;
    Path := Loc + FileMask;
    {Searches in current folder}
    if not Terminated and (FindFirst(Path, SearchAttr, SR) = 0) then
      repeat
        if (SR.Name[1] <> '.') and FindFile.AcceptFile(Loc, SR) then
        begin
          Folder := Loc;
          FoundSR := SR;
          Synchronize(FileFound);
        end;
      until Terminated or (FindNext(SR) <> 0);
    FindClose(SR);
    {Scans sub folders}
    if not Terminated and FindFile.SI.IncSubfolders then
    begin
      Path := Loc + '*.*';
      if not Terminated and (FindFirst(Path, SearchAttr, SR) = 0) then
        repeat
          if ((SR.Attr and faDirectory) <> 0) and (SR.Name[1] <> '.') then
            SearchIn(Loc + SR.Name + '\', FileMask);
        until Terminated or (FindNext(SR) <> 0);
      FindClose(SR);
    end;
  end;

var
  I: Integer;
  Folder, Filename: String;

begin
  try
    for I := 0 to FindFile.SI.IncludeFiles.Count-1 do
    begin
      Folder := ExpandFileName(ExtractFilePath(FindFile.SI.IncludeFiles[I]));
      Filename := ExtractFileName(FindFile.SI.IncludeFiles[I]);
      SearchIn(Folder, Filename);
      if FindFile.fAborted then Break;
    end;
  except
    FindFile.fAborted := True;
  end;
end;

{$ENDIF}

function TDFWFindFile.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWFindFile.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWFindFile.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWFindFile.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.

