{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWRarBar.pas                                           }
{ Short description  :                                                         }
{ Author             : Ales Berka                                              }
{ E-Mail             : xberka1@informatics.muni.cz                             }
{ Internet           : http://bealsoft.cjb.net                                 }
{ Version            : 1.1                                                     }
{ Date last modified :                                                         }
{ Delphi versions    : 2,3,4,5                                                 }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   : 1.1 - rewritten drawing method (floodfill no more!)     }
{                          - new properties - X, Y - position of bar, ...      }
{                          - Colorschemes                                      }
{                          - different Colours of frame under and above liquid }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWRarBar                                                }
{ Classes          :                                                           }
{ Types            : TRBColorScheme                                            }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWRarBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
      ComponentVersion='1.1';
      ComponentAuthor='Ales Berka';
      ComponentEMail='xberka1@informatics.muni.cz';
      ComponentHomePage='http://bealsoft.cjb.net';

type
  TRBColorScheme = (rcPurpleBlue, rcAcid, rcLightGreen, rcLightBlue);

  TDFWRarBar = class(TGraphicControl)
  private
    { Private declarations }
    FTopColor, FSideColor1, FSideColor2, FEmptyColor1, FEmptyColor2,
    FEmptyFrameColor1, FEmptyFrameColor2, FBottomFrameColor, FBottomColor,
    FFilledFrameColor, FFilledColor, FFilledSideColor1, FFilledSideColor2,
    FEmptyLineColor, FFilledLineColor : TColor;
    FMin, FMax, FProgress : Integer;
    FFont                 : TFont;
    FShowPerc             : Boolean;
    FColorScheme          : TRBColorScheme;
//    FTransparent          : Boolean;

    TopX,TopY             : Integer;
    Size                  : Integer;

    procedure Draw;
    procedure SetFont(Value : TFont);
    procedure SetMin(Value : Integer);
    procedure SetMax(Value : Integer);
    procedure SetProgress(Value : Integer);
    procedure SetShowPerc(Value : Boolean);
    procedure SetScheme(Value : TRBColorScheme);
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
//    procedure SetTransparent(Value : Boolean);

  protected
    { Protected declarations }
    procedure Paint; override;

  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    { Published declarations }
    property TopColor : TColor
      read FTopColor write FTopColor;
    property SideColor1 : TColor
      read FSideColor1 write FSideColor1;
    property SideColor2 : TColor
      read FSideColor2 write FSideColor2;
    property EmptyColor1 : TColor
      read FEmptyColor1 write FEmptyColor1;
    property EmptyColor2 : TColor
      read FEmptyColor2 write FEmptyColor2;
    property EmptyFrameColor1 : TColor
      read FEmptyFrameColor1 write FEmptyFrameColor1;
    property EmptyFrameColor2 : TColor
      read FEmptyFrameColor2 write FEmptyFrameColor2;
    property BottomFrameColor : TColor
      read FBottomFrameColor write FBottomFrameColor;
    property BottomColor : TColor
      read FBottomColor write FBottomColor;
    property FilledFrameColor : TColor
      read FFilledFrameColor write FFilledFrameColor;
    property FilledColor : TColor
      read FFilledColor write FFilledColor;
    property FilledSideColor1 : TColor
      read FFilledSideColor1 write FFilledSideColor1;
    property FilledSideColor2 : TColor
      read FFilledSideColor2 write FFilledSideColor2;
    property EmptyLineColor : TColor
      read FEmptyLineColor write FEmptyLineColor;
    property FilledLineColor : TColor
      read FFilledLineColor write FFilledLineColor;

    property Min : Integer
      read FMin write SetMin;
    property Max : Integer
      read FMax write SetMax;
    property Progress : Integer
      read FProgress write SetProgress;
    property Font : TFont
      read FFont write SetFont;
    property ShowPercentage : Boolean
      read FShowPerc write SetShowPerc;
    property X : Integer
      read TopX write TopX;
    property Y : Integer
      read TopY write TopY;
    property ColorScheme : TRBColorScheme
      read FColorScheme write SetScheme;
//    property Transparent : Boolean
//      read FTransparent write SetTransparent;

    property Color;

  end;

implementation

uses ExtCtrls;

constructor TDFWRarBar.Create(AOwner : TComponent);
begin
  inherited;

  FFont:=TFont.Create;

  SetScheme(rcPurpleBlue);

  FShowPerc:=True;
  FMin:=0;
  FMax:=100;
  FProgress:=0;
  TopX:=0;
  TopY:=5;
//  FTransparent:=False;

  Height:=180;
  Width:=60;
  Color:=clBtnFace;
  FFont.Style:=[fsBold];
end;

destructor TDFWRarBar.Destroy;
begin
  FFont.Free;

  inherited;
end;

procedure TDFWRarBar.SetFont(Value : TFont);
begin
  FFont.Assign(Value);
end;

procedure TDFWRarBar.Paint;
begin
  inherited;
  Draw;
end;

procedure TDFWRarBar.Draw;

var Points : Array[1..4] of TPoint;
    MemI   : TImage;
    Prog   : Integer;
    R      : Real;
    Perc   : Integer;
    S      : String;

procedure DrawFrame(Canvas : TCanvas);
begin
  with Canvas do
  begin
    Pen.Width:=1; Pen.Style:=psSolid; Pen.Mode:=pmCopy;

    MoveTo(TopX,TopY+5);
    Pen.Color:=FEmptyLineColor;
    LineTo(PenPos.X,PenPos.Y+(Size-10-prog));
    Pen.Color:=FFilledLineColor;
    LineTo(PenPos.X,PenPos.Y+prog);

    MoveTo(TopX+15,TopY+10);
    Pen.Color:=FEmptyLineColor;
    LineTo(PenPos.X,PenPos.Y+(Size-10-prog));
    Pen.Color:=FFilledLineColor;
    LineTo(PenPos.X,PenPos.Y+prog);

    MoveTo(TopX+30,TopY+5);
    Pen.Color:=FEmptyLineColor;
    LineTo(PenPos.X,PenPos.Y+(Size-10-prog));
    Pen.Color:=FFilledLineColor;
    LineTo(PenPos.X,PenPos.Y+prog);


    MoveTo(TopX,TopY+5);
    if (size-10=prog) then
        Pen.Color:=FFilledLineColor
    else
        Pen.Color:=FEmptyLineColor;
    MoveTo(TopX,TopY+5);
    LineTo(PenPos.X+15,PenPos.Y-5);
    LineTo(PenPos.X+15,PenPos.Y+5);
    LineTo(PenPos.X-15,PenPos.Y+5);
    LineTo(PenPos.X-15,PenPos.Y-5);

    MoveTo(TopX,size-5+TopY);
    if (prog<>0) then
        Pen.Color:=FFilledLineColor
    else
        Pen.Color:=FEmptyLineColor;
    LineTo(PenPos.X+15,PenPos.Y+5);
    LineTo(PenPos.X+15,PenPos.Y-5);

  end;
end;

begin
  Size:=Height-TopY-5;
  if (Size = 0) or ((FMax - FMin) = 0) then
  begin
    Perc:=0;
    Prog:=0;
  end else
  begin
    R:=(Progress - FMin) / ( (FMax - FMin) / (Size-10));
    Prog:=Round(R);
    Perc:=Round(R / ((Size-10) / 100));
  end;

  if Prog<0 then Prog:=0 else
    if (Prog>Size-10) then Prog:=Size-10;

  MemI:=TImage.Create(Self);
  MemI.Width:=Width;
  MemI.Height:=Height;
//  if FTransparent then
//    MemI.Transparent:=True else
//    MemI.Transparent:=False;

  with MemI.Canvas do
  begin
(* Transparency
    if not FTransparent then
    begin *)
    Brush.Color:=Color;
    FillRect(ClipRect);

    DrawFrame(MemI.Canvas);

    Brush.Color:=TopColor;
    Points[1]:=Point(TopX+15,TopY+10);
    Points[2]:=Point(TopX,TopY+5);
    Points[3]:=Point(TopX+15,TopY);
    Points[4]:=Point(TopX+(2*15),TopY++5);
    Polygon(Points);

    Brush.Color:=SideColor1;
    Points[1]:=Point(TopX+15,TopY+Size);
    Points[2]:=Point(TopX,TopY+Size-5);
    Points[3]:=Point(TopX,TopY+5);
    Points[4]:=Point(TopX+15,TopY+10);
    Polygon(Points);

    Brush.Color:=SideColor2;
    Points[1]:=Point(TopX+15,TopY+Size);
    Points[2]:=Point(TopX+(15*2),TopY+Size-5);
    Points[3]:=Point(TopX+(15*2),TopY+5);
    Points[4]:=Point(TopX+15,TopY+10);
    Polygon(Points);

    if (Prog>0) then
    begin
      Brush.Color:=FilledSideColor1;
      Pen.Color:=FilledFrameColor;
      Points[1]:=Point(TopX+15,TopY+Size);
      Points[2]:=Point(TopX,TopY+Size-5);
      Points[3]:=Point(TopX,TopY+Size-Prog-5);
      Points[4]:=Point(TopX+15,TopY+Size-Prog-(2*5));
      Polygon(Points);

      Brush.Color:=FilledSideColor2;
      Pen.Color:=FilledFrameColor;
      Points[1]:=Point(TopX+15,TopY+Size);
      Points[2]:=Point(TopX+(15*2),TopY+Size-5);
      Points[3]:=Point(TopX+(15*2),TopY+Size-Prog-5);
      Points[4]:=Point(TopX+15,TopY+Size-Prog-(2*5));
      Polygon(Points);

      MoveTo(TopX,TopY+Size-5);
      Pen.Color:=BottomFrameColor;
      Brush.Color:=BottomColor;
      Points[1]:=Point(TopX+15,TopY+Size);
      Points[2]:=Point(TopX,   TopY+Size-5);
      Points[3]:=Point(TopX+15,TopY+Size-10);
      Points[4]:=Point(TopX+(15*2),TopY+Size-5);
      Polygon(Points);

      Brush.Color:=FilledColor;
      Pen.Color:=FilledFrameColor;

      Points[1]:=Point(TopX+15,TopY+Size-Prog);
      Points[2]:=Point(TopX,   TopY+Size-Prog-5);
      Points[3]:=Point(TopX+15,TopY+Size-Prog-(2*5));
      Points[4]:=Point(TopX+(15*2),TopY+Size-Prog-5);
      Polygon(Points);

      DrawFrame(MemI.Canvas);
    end else
    begin
      {EMPTY}
      Brush.Color:=EmptyColor1;
      Pen.Color:=EmptyFrameColor1;
      Points[1]:=Point(TopX+15,TopY+Size);
      Points[2]:=Point(TopX,   TopY+Size-5);
      Points[3]:=Point(TopX+15,TopY+Size-10);
      Points[4]:=Point(TopX+15,TopY+Size-5);
      Polygon(Points);


      Brush.Color:=EmptyColor2;
      Pen.Color:=EmptyFrameColor2;
      Points[1]:=Point(TopX+15,TopY+Size);
      Points[2]:=Point(TopX+30,TopY+Size-5);
      Points[3]:=Point(TopX+15,TopY+Size-10);
      Points[4]:=Point(TopX+15,TopY+Size-5);
      Polygon(Points);

      DrawFrame(MemI.Canvas);
    end;
    if FShowPerc then
    begin
//      if FTransparent then Brush.Color:=clNone else
      Brush.Color:=Color;
      S:=IntToStr(Perc)+'%';
      MemI.Canvas.Font.Assign(FFont);
      MemI.Canvas.TextOut(TopX+(2*15)+3,TopY+Size-Prog-TextHeight(S),S);
    end;
  end;
(* // Transparency
  if Transparent then
  begin
    Canvas.Brush.Color:=Canvas.Pixels[0,0];
    Canvas.FillRect(Canvas.ClipRect);
  end else *)

  Canvas.Draw(0,0,MemI.Picture.Graphic);
  MemI.Free;
end;

procedure TDFWRarBar.SetMin(Value : Integer);
begin
  FMin:=Value;
  Draw;
end;

procedure TDFWRarBar.SetMax(Value : Integer);
begin
  FMax:=Value;
  Draw;
end;

procedure TDFWRarBar.SetProgress(Value : Integer);
begin
  FProgress:=Value;
  Draw;
end;

procedure TDFWRarBar.SetShowPerc(Value : Boolean);
begin
  FShowPerc:=Value;
  Draw;
end;

procedure TDFWRarBar.SetScheme(Value : TRBColorScheme);
begin
  FColorScheme:=Value;
  case FColorScheme of
    rcPurpleBlue : begin
                     FEmptyLineColor:=$FFB0B0;
                     FFilledLineColor:=$D060E0;
                     FTopColor:=$FF8080;
                     FSideColor1:=$E06868;
                     FSideColor2:=$FF8080;
                     FEmptyFrameColor1:=$A06868;
                     FEmptyFrameColor2:=$BF8080;
                     FEmptyColor1:=$C06868;
                     FEmptyColor2:=$DF8080;
                     FBottomFrameColor:=$64408C;
                     FBottomColor:=$7A408C;
                     FFilledFrameColor:=$8060A0;
                     FFilledSideColor1:=$823C96;
                     FFilledSideColor2:=$8848C0;
                     FFilledColor:=$A060A0;
                     FFont.Color:=clPurple;
                   end;
    rcAcid       : begin
                     FEmptyLineColor:=$E0FFE0;
                     FFilledLineColor:=$68A8E0;
                     FTopColor:=$80FF80;
                     FSideColor1:=$68E068;
                     FSideColor2:=$80FF80;
                     FEmptyFrameColor1:=$68A068;
                     FEmptyFrameColor2:=$80BF80;
                     FEmptyColor1:=$68C068;
                     FEmptyColor2:=$80DF80;
                     FBottomFrameColor:=$40648C;
                     FBottomColor:=$407A8C;
                     FFilledFrameColor:=$60A0A0; //$6080A0;
                     FFilledSideColor1:=$3C8296;
                     FFilledSideColor2:=$4888C0;
                     FFilledColor:=$60A0A0;
                     FFont.Color:=clAqua;
                   end;
    rcLightGreen : begin
                     FEmptyLineColor:=$FFE0E0;
                     FFilledLineColor:=$98E068;
                     FTopColor:=$FF8080;
                     FSideColor1:=$E06868;
                     FSideColor2:=$FF8080;
                     FEmptyFrameColor1:=$A06868;
                     FEmptyFrameColor2:=$BF8080;
                     FEmptyColor1:=$C06868;
                     FEmptyColor2:=$DF8080;
                     FBottomFrameColor:=$648C40;
                     FBottomColor:=$7A8C40;
                     FFilledFrameColor:=$80A060;
                     FFilledSideColor1:=$82963C;
                     FFilledSideColor2:=$88C048;
                     FFilledColor:=$A0A060;
                     FFont.Color:=$489008;
                   end;
    rcLightBlue  : begin
                     FEmptyLineColor:=$C8E988;
                     FFilledLineColor:=$D0A8A8;
                     FTopColor:=$88C048;
                     FSideColor1:=$82963C;
                     FSideColor2:=$88C048;
                     FEmptyFrameColor1:=$A06868;
                     FEmptyFrameColor2:=$BF8080;
                     FEmptyColor1:=$82963C;
                     FEmptyColor2:=$88C048;
                     FBottomFrameColor:=$D05858;
                     FBottomColor:=$E05858;
                     FFilledFrameColor:=$D08080;
                     FFilledSideColor1:=$E06868;
                     FFilledSideColor2:=$FF8080;
                     FFilledColor:=$E08080;
                     FFont.Color:=clPurple;
                   end;
  end;
  Invalidate;
end;


(* Transparency
procedure TDFWRarBar.SetTransparent(Value : Boolean);
begin
  FTransparent:=Value;
  Draw;
end;
*)

function TDFWRarBar.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWRarBar.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWRarBar.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWRarBar.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
