{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWSelectDirView.pas                                    }
{ Short description  :                                                         }
{ Author             : Jonathan Faivre-Vuillin                                 }
{ E-Mail             : Lapince@Ifrance.com                                     }
{ Internet           :                                                         }
{ Version            : 2.0                                                     }
{ Date last modified : 12/04/2000                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWSelectDirView                                         }
{ Classes          :                                                           }
{ Types            : TDFWCheckMark                                             }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSelectDirView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ShellAPI, FileCtrl, DsgnIntf, ImgList,stdctrls;

const
      ComponentVersion='2.0';
      ComponentAuthor='Jonathan Faivre-Vuillin';
      ComponentEMail='Lapince@Ifrance.com';
      ComponentHomePage='';

type
  TDFWCheckMark=(cmChecked, cmGrayed, cmUnchecked,cmUniqueChecked,cmUniqueGrayed);

type
  TDFWSelectDirView = class(TTreeView)
  private
    FGrayed,FChecked,FUnChecked, FUnique, FUniqueGrayed: TBitmap;
    FAddBeforeExpand: Boolean;
    FActive: Boolean;
    FDrive: Char;
    FIconList: TImageList;
    FCheckList: TCustomImageList;
    FCheckOnlyOnClick: Boolean;
    DownNode: TTreeNode;
    procedure OnTreeExpanded (Sender: TObject; Node: TTreeNode);
    procedure OnTreeExpanding (Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure OnNodeChange (Sender: TObject; Node: TTreeNode);
    procedure OnNodeMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OnNodeMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  protected
    FlagsUnchecking: Boolean;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    function AddSubDirs(TreeView: TDFWSelectDirView; ParentNode: TTreeNode; Directory: String) : boolean;
    function HasSubDir(Path: String): Boolean;
    procedure SetDrive(Value: Char);
    procedure ControlParentState(Parent: TTreeNode);
    procedure GetSelected(Node: TTreeNode; var List: TStringList);
  public
    procedure Reload;
    procedure createwnd ; override;
    constructor Create(AOwner: TComponent); override;
    destructor destroy; override;
    procedure SetNodeState(Node: TTreeNode; Check: TDFWCheckMark);  // Change the specified node check state and update parents & child nodes.
    function NodeState(Node: TTreeNode): TDFWCheckMark;        // Returns the specified node check state
    function GetNodeDirectory (Node: TTreeNode): string;        // Returns the specified node directory
    Procedure GetSelection(var List: TStringList);                         // Retruns all directory selected in the TreeView
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    Property Active: Boolean read FActive write FActive;
    Property Drive: Char read FDrive write SetDrive;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property HideSelection;
    property OnEditing;
    property OnEdited;
    property OnExpanding;
    property OnExpanded;
    property OnCollapsing;
    property OnCollapsed;
    property OnChanging;
    property OnChange;
    property OnDeletion;
    property Align;
    property Enabled;
    property Font;
    property Color;
    property ParentColor;
    property ParentCtl3D;
    property Ctl3D;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property PopupMenu;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;

    property CheckOlnyOnClick: Boolean read FCheckOnlyOnClick write FCheckOnlyOnClick;
//    property AutoScanOnLoad: Boolean read FAutoScanOnLoad write FAutoScanOnLoad;
    property AddBeforeExpand: Boolean read FAddBeforeExpand write FAddBeforeExpand;
//    property UncheckParentIfNoChildChecked: Boolean read FUncheckParentIfNoChildChecked write FUncheckParentIfNoChildChecked;
  end;

type
  TDFWNodeInfo=class(TObject)
  public
    Directory: String;
    Visited: Boolean;
    Opened: Boolean;

    Unique: Boolean;
    State: TDFWCheckMark;
  end;

implementation

{$R DFWCHECKBITMAPS.RES}

procedure TDFWSelectDirView.GetSelection(var List: TStringList);
var
  I: Integer;
  NodeInfo: TDFWNodeInfo;
begin
  NodeInfo:=Items[0].Data;
  if NodeInfo.State=cmChecked then
  begin
    List.Add(NodeInfo.Directory+'*');
    Exit;
  end
  else if NodeInfo.State=cmGrayed then
    List.Add(NodeInfo.Directory);

    GetSelected(Items[0],List);
//  for I:=0 to Items[0].Count-1 do
//    GetSelected(Items[0].Item[I],List);
end;

procedure TDFWSelectDirView.GetSelected(Node: TTreeNode; var List: TStringList);
var
  I: Integer;
  NodeInfo: TDFWNodeInfo;
begin
  for I:=0 to Node.Count-1 do
  begin
    NodeInfo:=Node.Item[I].Data;
    if (NodeInfo.State=cmChecked) or (NodeInfo.State=cmUniqueChecked) then
      List.Add(NodeInfo.Directory+'*')
    else if (NodeInfo.State=cmGrayed) or (NodeInfo.State=cmUniqueGrayed) then
    begin
      List.Add(NodeInfo.Directory);
      GetSelected(Node.Item[I],List);
    end
    else
      if NodeInfo.visited=True then
        GetSelected(Node.Item[I],List);
  end;
end;

function TDFWSelectDirView.GetNodeDirectory (Node: TTreeNode): string;
var
  NodeInfo: TDFWNodeInfo;
begin
  NodeInfo:=Node.Data;
  Result:=NodeInfo.Directory;
end;

function TDFWSelectDirView.NodeState (Node: TTreeNode): TDFWCheckMark;
var
  NodeInfo: TDFWNodeInfo;
begin
  NodeInfo:=Node.Data;
  Result:=NodeInfo.State;
end;

procedure TDFWSelectDirView.SetNodeState(Node: TTreeNode; Check: TDFWCheckMark);
var
  I: Integer;
  NodeInfo,ParentInfo: TDFWNodeInfo;
  Unique: Boolean;
  SaveState: TDFWCheckMark;
begin
  NodeInfo:=Node.Data;
  Unique:=NodeInfo.Unique;
  //Si le nouvel tat est coch
  if NodeInfo.Visited=True then
    for I:=0 to Node.Count-1 do                          //Mettre tous les noeuds enfants au mme tat de coche que leur parent
      if Check=cmChecked then
        SetNodeState(Node.Item[I],cmChecked)
      else if Check=cmUnchecked then
        SetNodeState(Node.Item[I],cmUnChecked)
      else if Check=cmUniqueChecked then
        SetNodeState(Node.Item[I],cmUniqueChecked)
      else if Check=cmUniqueChecked then
        SetNodeState(Node.Item[I],cmUniqueChecked);

  if Check=cmChecked then
  begin
    NodeInfo.State:=cmChecked;
    Node.StateIndex:=1;
    if Node.Parent=nil then Exit;
    ControlParentState(Node.Parent);
  end;

  //Si le nouvel tat est dcoch ou gris...
  if (Check=cmUnchecked) or (Check=cmGrayed) then
  begin
    NodeInfo.State:=Check;                               //Mettre l'tat  non coch ou gris
    Node.StateIndex:=2;
    if Node.Parent=nil then Exit;                        //Quitter s'il n'y a pas de parent
    ParentInfo:=Node.Parent.Data;
    if (ParentInfo.State=cmUnchecked) or (ParentInfo.State=cmGrayed) then     //Si le parent est gris ou dcoch, refaire l'analyse
      ControlParentState(Node.Parent)
    else                                                                      //sinon...
    begin
      if Node.Parent.Count>1 then                                             
      begin
        if ParentInfo.Unique then                                             //Si le parent a plus d'un enfant, comme il tait coch, on le dcoche simplement...
        begin
          ParentInfo.State:=cmUniqueGrayed;
          Node.Parent.StateIndex:=5;
        end
        else
        begin
          ParentInfo.State:=cmGrayed;
          Node.Parent.StateIndex:=3;
        end;
        if Node.Level>1 then
          ControlParentState(Node.Parent.Parent)                              //... et mettre  jour les parents du parent s'il en a
      end
      else
      begin
        Node.Parent.StateIndex:=2;                                            //Si le parent n'a qu'un enfant, le dcocher...
        ParentInfo.State:=cmUnchecked;
        if Node.Level>1 then
          ControlParentState(Node.Parent.Parent)                              //... et mettre  jour les parents du parent s'il en a
      end;
    end;
  end;

  //Si le nouvel tat est Unique
  if Check=cmUniqueChecked then
  begin
    if Node.Level=0 then   //Si le noeud est de niveau 0 il ne peut pas tre "unique"
    begin
      SetNodeState(Node,cmChecked);
      Exit;
    end;
    ParentInfo:=Node.Parent.Data;
    if ((ParentInfo.State=cmChecked) or (ParentInfo.State=cmGrayed)) and not FlagsUnchecking then  //Si le parent est gris ou coch, l'enfant ne peut pas tre "unique"
    begin                                                                                  //m^ chose si le noeud a un niveau 0
      SetNodeState(Node,cmChecked);
      Exit;
    end;
    NodeInfo.State:=cmUniqueChecked;
    NodeInfo.Unique:=True;
    Node.StateIndex:=4;
    ControlParentState(Node.Parent)
  end;
end;

procedure TDFWSelectDirView.ControlParentState(Parent: TTreeNode);
var
  I: Integer;
  NodeInfo: TDFWNodeInfo;
  ParentState: TDFWCheckMark;
  Done: Boolean;
  HasUnique: Boolean;
begin
    Done:=False;
    for I:=0 to Parent.Count-1 do
    begin
      NodeInfo:=Parent.Item[I].Data;
      if NodeInfo.State=cmGrayed then
      begin
        ParentState:=cmGrayed;
        break;
      end;
      if (NodeInfo.State=cmUnchecked) then
        if Done=False then
        begin
          ParentState:=cmUnchecked;
          Done:=true
        end
        else
          if ParentState=cmChecked then
          begin
            ParentState:=cmGrayed;
            break;
          end;
      if (NodeInfo.State=cmChecked) then
        if Done=False then
        begin
          ParentState:=cmChecked;
          Done:=true
        end
        else
          if ParentState=cmUnchecked then
          begin
            ParentState:=cmGrayed;
            break;
          end;
    end;
    NodeInfo:=Parent.Data;
    if NodeInfo.State<>ParentState then
    begin
      if NodeInfo.Unique and HasUnique then
      begin
        case ParentState of
          cmChecked: Parent.StateIndex:=4;
          cmGrayed: Parent.StateIndex:=5;
          cmUnchecked: Parent.StateIndex:=2;
        end;
      end
      else
      begin
        case ParentState of
          cmChecked: Parent.StateIndex:=1;
          cmGrayed: Parent.StateIndex:=3;
          cmUnchecked: Parent.StateIndex:=2;
        end;
      end;
      if (ParentState=cmChecked) or (ParentState=cmGrayed) then
        if HasUnique then
        begin
          for I:=0 to Parent.Count-1 do
          begin
            NodeInfo:=Parent.item[I].Data;
            if NodeInfo.State=cmUniqueChecked then
              SetNodeState(Parent.item[I],cmChecked);
          end;
        end;
      NodeInfo:=Parent.Data;
      NodeInfo.State:=ParentState;
      if Parent.Level<>0 then
        ControlParentState(Parent.Parent);
    end;
end;



constructor TDFWSelectDirView.Create(AOwner: TComponent);
var
  Sfi: TshFileInfo;
begin
  Inherited Create(AOwner);
  FGrayed:=TBitmap.Create;
  FChecked:=TBitmap.Create;
  FUnChecked:=TBitmap.Create;
  FUnique:=TBitmap.Create;
  FUniqueGrayed:=TBitmap.Create;

  FChecked.Width:=17;
  FChecked.Height:=17;
  FUnChecked.Width:=17;
  FUnChecked.Height:=17;
  FGrayed.Width:=17;
  FGrayed.Height:=17;
  FUnique.Width:=17;
  FUnique.Height:=17;

  FGrayed.Handle:=LoadBitmap(HInstance,'GRAYED');
  FChecked.Handle:=LoadBitmap(HInstance,'CHECKED');
  FUnChecked.Handle:=LoadBitmap(HInstance,'UNCHECKED');
  FUnique.Handle:=LoadBitmap(HInstance,'UNIQUE');
  FUniqueGrayed.Handle:=LoadBitmap(HInstance,'UNIQUEGRAYED');

  FCheckList:=TImageList.CreateSize(17,17);
  FCheckList.AddMasked(FUnChecked,clFuchsia);
  FCheckList.AddMasked(FChecked,clFuchsia);        //1 : Coch
  FCheckList.AddMasked(FUnChecked,clFuchsia);      //2 : Dcoch
  FCheckList.AddMasked(FGrayed,clFuchsia);         //3 : Gris
  FCheckList.AddMasked(FUnique,clFuchsia);         //4 : Croix
  FCheckList.AddMasked(FUniqueGrayed,clFuchsia);   //5 : Croix grise

  OnExpanding:=OnTreeExpanding;
  OnExpanded:=OnTreeExpanded;
  OnChange:=OnNodeChange;
  OnMouseDown:=OnNodeMouseDown;
  OnMouseUp:=OnNodeMouseUp;

  FIconList:=TImageList.Create(self);
  try
    FIconList.handle :=shgetfileinfo('',0,sfi,sizeof(tshfileinfo),shgfi_sysiconindex or shgfi_smallicon);
    FIconList.shareimages := true;
  except;
  end;

  ReadOnly:=True;
  Images:=FIconList;
  StateImages:=FCheckList;
  FActive:=true;
end;

destructor TDFWSelectDirView.Destroy;
begin
  FGrayed.Free;
  FChecked.Free;
  FUnChecked.Free;
  FCheckList.Free;
  FUnique.Free;
  FUniqueGrayed.Free;
  inherited;
end;

procedure TDFWSelectDirView.OnNodeMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
  NodeInfo, ParentInfo: TDFWNodeInfo;
  Check: TDFWCheckMark;
begin
  if Button=mbRight then
  begin
    if not (htOnStateIcon in GetHitTestInfoAt(X,Y)) then
      inherited;
  end
  else
    inherited;
  Node:=GetNodeAt(X,Y);
  if Node=nil then Exit;
  if htOnStateIcon in GetHitTestInfoAt(X,Y) then
  begin
    if not FCheckOnlyOnClick then
    begin
      NodeInfo:=Node.Data;
      DownNode:=nil;
      if button=mbLeft then
      begin
        if (NodeInfo.State=cmGrayed) or (NodeInfo.State=cmChecked) then
          Check:=cmUnchecked
        else
          Check:=cmChecked;
        SetNodeState(Node,Check);
      end
      else if button=mbRight then
      begin
        if Node.Level=0 then
        begin
          SetNodeState(Node,cmChecked);
          Exit;
        end;
        ParentInfo:=Node.Parent.Data;
        if (ParentInfo.State=cmChecked) or (ParentInfo.State=cmGrayed) then
          Check:=cmChecked
        else if NodeInfo.State=cmUnChecked then
          Check:=cmUniqueChecked
        else if NodeInfo.State=cmUniqueChecked then
          Check:=cmUnchecked
        else if NodeInfo.State=cmUniqueGrayed then
          Check:=cmUnchecked;
        SetNodeState(Node,Check);
      end;
    end
    else
      DownNode:=Node;
  end;
end;

procedure TDFWSelectDirView.OnNodeMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
  NodeInfo,ParentInfo: TDFWNodeInfo;
  Check: TDFWCheckMark;
begin
  inherited;
  if not FCheckOnlyOnClick then Exit;
  if Button=mbRight then
  begin
    if not (htOnStateIcon in GetHitTestInfoAt(X,Y)) then
      inherited;
  end
  else
    inherited;
  Node:=GetNodeAt(X,Y);
  if Node=nil then Exit;
  if htOnStateIcon in GetHitTestInfoAt(X,Y) then
  begin
    NodeInfo:=Node.Data;
    DownNode:=nil;
    if button=mbLeft then
    begin
      if (NodeInfo.State=cmGrayed) or (NodeInfo.State=cmChecked) then
        Check:=cmUnchecked
      else
        Check:=cmChecked;
      SetNodeState(Node,Check);
    end
    else if button=mbRight then
    begin
      if Node.Level=0 then
      begin
        SetNodeState(Node,cmChecked);
        Exit;
      end;
      ParentInfo:=Node.Parent.Data;
      if (ParentInfo.State=cmChecked) or (ParentInfo.State=cmGrayed) then
        Check:=cmChecked
      else if NodeInfo.State=cmUnChecked then
        Check:=cmUniqueChecked
      else if NodeInfo.State=cmUniqueChecked then
        Check:=cmUnchecked
      else if NodeInfo.State=cmUniqueGrayed then
        Check:=cmUnchecked;
      SetNodeState(Node,Check);
    end;
  end;
end;

procedure TDFWSelectDirView.OnNodeChange(Sender: TObject; Node: TTreeNode);
var
  NodeInfo: TDFWNodeInfo;
  sfi: TshFileInfo;
begin
  inherited;
  NodeInfo:=Node.Data;
  If NodeInfo.Opened=false then
  begin
    shgetfileinfo(PChar(NodeInfo.Directory),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon or shgfi_openicon);
    Node.selectedindex := sfi.iicon;
  end;
end;

procedure TDFWSelectDirView.SetDrive(Value: Char);
var
  NodeInfo: TDFWNodeInfo;
  Node: TTreeNode;
  sfi: TShFileInfo;
begin
  If (Value in ['a'..'z','A'..'Z']) then
  begin
    Items.Clear;
    FDrive:=Value;
    if not HandleAllocated then HandleNeeded;
    if FActive then
      if (not (csLoading in ComponentState)) and (not(csDesigning in ComponentState)) then
      begin
        SHGetFileInfo(PChar(FDrive+':\'),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon or shgfi_displayname);
        Node:=Self.Items.Add(nil,sfi.szDisplayName);
        Node.StateIndex:=1;
        NodeInfo:=TDFWNodeInfo.Create;
        NodeInfo.Directory:=FDrive+':\';
        NodeInfo.Visited:=true;
        Node.Data:=NodeInfo;
        Node.ImageIndex:=sfi.iIcon;
        Node.SelectedIndex:=sfi.iIcon;
        NodeInfo.State:=cmChecked;
        AddSubDirs(self, node,FDrive+':\');
      end;
  end;
end;

procedure TDFWSelectDirView.Reload;
begin
  SetDrive(FDrive);
end;

procedure TDFWSelectDirView.OnTreeExpanding (Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
var
  T: String;
  NodeInfo: TDFWNodeInfo;
begin
  Inherited;
  if FAddBeforeExpand then
  begin
    NodeInfo:=Node.Data;
    If not NodeInfo.Visited then
    begin
         T:='';
         Node.Item[0].delete;
         NodeInfo:=Node.Data;
         T:=NodeInfo.Directory;
         AddSubDirs(Self,Node,T);
         NodeInfo.Visited:=true;
    end;
  end;
end;

procedure TDFWSelectDirView.OnTreeExpanded (Sender: TObject; Node: TTreeNode);
var
  T: string;
  NodeInfo: TDFWNodeInfo;
begin
  Inherited;
  if not FAddBeforeExpand then
  begin
    NodeInfo:=Node.Data;
    If not NodeInfo.Visited then
    begin
      Node.Item[0].delete;
      NodeInfo:=Node.Data;
      T:=NodeInfo.Directory;
      AddSubDirs(Self,Node,T);
      NodeInfo.Visited:=true;
    end;
  end;
end;

function TDFWSelectDirView.HasSubDir(Path: String): Boolean;
var
  SearchRec: TSearchRec;
  Found: Integer;
begin
  Result:=false;
  Found := FindFirst(Path+'*.*', faDirectory, SearchRec);
  while Found=0 do
  begin
    If ((SearchRec.Attr and faDirectory) > 0) and (SearchRec.name[1] <> '.') then
      If SearchRec.Name<>'' then
      begin
        Result:=true;
        exit;
      end;
    Found := FindNext(SearchRec);
  end;
end;

function TDFWSelectDirView.AddSubDirs(TreeView: TDFWSelectDirView; ParentNode: TTreeNode; Directory: String) : boolean;
var
  SearchRec: TSearchRec;
  DirList: TStringList;
  Found: Integer;
  FoundOne: Boolean;
  Node: TTreeNode;
  sfi: TshFileInfo;
  I: Integer;
  CompS: String;
  NodeInfo,ParentInfo: TDFWNodeInfo;
begin
  DirList:=TStringList.Create;
  DirList.Sorted:=true;

  If Directory[length(Directory)]<>'\' then Directory:=Directory+'\';

  Found := FindFirst(Directory+'*.*', faDirectory, SearchRec);
  while Found=0 do
  begin
    FoundOne:=True;
    If ((SearchRec.Attr and faDirectory) > 0) and (SearchRec.name[1] <> '.') then
      If (DirectoryExists(Directory+SearchRec.Name)) and (SearchRec.Name<>'') then
        DirList.Add(SearchRec.Name);
    Found := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
  I:=0;
  if FoundOne then
  begin
    SHGetFileInfo(PChar(Directory+DirList[I]),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon or shgfi_displayname);
    Node:=TreeView.Items.AddChildFirst(ParentNode,sfi.szDisplayName);
    NodeInfo:=TDFWNodeInfo.Create;
    ParentInfo:=ParentNode.Data;
    case ParentInfo.State of
      cmChecked: Node.StateIndex:=1;
      cmGrayed: Node.StateIndex:=2;
      cmUnchecked: Node.StateIndex:=2;
      cmUniqueChecked: Node.StateIndex:=4;
      cmUniqueGrayed: Node.StateIndex:=5;
    end;
    NodeInfo.Unique:=ParentInfo.unique;
    NodeInfo.State:=ParentInfo.State;
    NodeInfo.Directory:=Directory+DirList[I]+'\';
    Node.ImageIndex:=sfi.iicon;
    shgetfileinfo(PChar(Directory+DirList[I]),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon or shgfi_openicon);
    Node.selectedindex := sfi.iicon;
    Node.Data:=NodeInfo;
    If HasSubDir(NodeInfo.Directory) then
      TreeView.Items.AddChildFirst(Node,'');
  end;


  For I:=1 to DirList.Count -1 do
  begin
    SHGetFileInfo(PChar(Directory+DirList[I]),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon or shgfi_displayname);
    CompS:=sfi.szDisplayName;
    If CompS='' then
      continue;
    Node:=TreeView.Items.Add(ParentNode.Item[I-1],sfi.szDisplayName);
    NodeInfo:=TDFWNodeInfo.Create;
    ParentInfo:=ParentNode.Data;
    case ParentInfo.State of
      cmChecked: Node.StateIndex:=1;
      cmGrayed: Node.StateIndex:=2;
      cmUnchecked: Node.StateIndex:=2;
      cmUniqueChecked: Node.StateIndex:=4;
      cmUniqueGrayed: Node.StateIndex:=5;
    end;
    NodeInfo.State:=ParentInfo.State;
    NodeInfo.Directory:=Directory+DirList[I]+'\';
    NodeInfo.Unique:=ParentInfo.unique;
    Node.ImageIndex:=sfi.iicon;
    Node.Data:=NodeInfo;
    If HasSubDir(NodeInfo.Directory) then
       TreeView.Items.AddChildFirst(Node,'');
  end;
  Result:=true;
end;

procedure TDFWSelectDirView.createwnd ;
begin
  inherited;
  Reload;
end;

function TDFWSelectDirView.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSelectDirView.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSelectDirView.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSelectDirView.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
